"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFilesCrud = void 0;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _handlerGraphql = require("@webiny/handler-graphql");
var _apiSecurity = require("@webiny/api-security");
var _pubsub = require("@webiny/pubsub");
var _error = _interopRequireDefault(require("@webiny/error"));
var _checkBasePermissions = require("./checkBasePermissions");
/**
 * If permission is limited to "own" files only, check that current identity owns the file.
 */
const checkOwnership = (file, permission, identity) => {
  if ((permission === null || permission === void 0 ? void 0 : permission.own) === true) {
    if (file.createdBy.id !== identity.id) {
      throw new _apiSecurity.NotAuthorizedError();
    }
  }
};
const createFilesCrud = config => {
  const {
    storageOperations,
    getLocaleCode,
    getTenantId,
    getIdentity,
    getPermission,
    WEBINY_VERSION
  } = config;
  return {
    onFileBeforeCreate: (0, _pubsub.createTopic)("fileManager.onFileBeforeCreate"),
    onFileAfterCreate: (0, _pubsub.createTopic)("fileManager.onFileAfterCreate"),
    onFileBeforeBatchCreate: (0, _pubsub.createTopic)("fileManager.onFileBeforeBatchCreate"),
    onFileAfterBatchCreate: (0, _pubsub.createTopic)("fileManager.onFileAfterBatchCreate"),
    onFileBeforeUpdate: (0, _pubsub.createTopic)("fileManager.onFileBeforeUpdate"),
    onFileAfterUpdate: (0, _pubsub.createTopic)("fileManager.onFileAfterUpdate"),
    onFileBeforeDelete: (0, _pubsub.createTopic)("fileManager.onFileBeforeDelete"),
    onFileAfterDelete: (0, _pubsub.createTopic)("fileManager.onFileAfterDelete"),
    async getFile(id) {
      const permission = await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "r"
      });
      const file = await storageOperations.files.get({
        where: {
          id,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      });
      if (!file) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }
      checkOwnership(file, permission, getIdentity());
      return file;
    },
    async createFile(input, meta) {
      await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "w"
      });
      const identity = getIdentity();

      // Extract ID from file key
      const [id] = input.key.split("/");
      const file = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, input), {}, {
        tags: Array.isArray(input.tags) ? input.tags : [],
        aliases: Array.isArray(input.aliases) ? input.aliases : [],
        id: input.id || id,
        meta: (0, _objectSpread2.default)({
          private: false
        }, input.meta || {}),
        tenant: getTenantId(),
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          displayName: identity.displayName,
          type: identity.type
        },
        locale: getLocaleCode(),
        webinyVersion: WEBINY_VERSION
      });
      try {
        await this.onFileBeforeCreate.publish({
          file,
          meta
        });
        const result = await storageOperations.files.create({
          file
        });
        await this.onFileAfterCreate.publish({
          file,
          meta
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create a file.", ex.code || "CREATE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          file
        }));
      }
    },
    async updateFile(id, input) {
      const permission = await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "w"
      });
      const original = await storageOperations.files.get({
        where: {
          id,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      });
      if (!original) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }
      checkOwnership(original, permission, getIdentity());
      const file = (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, original), input), {}, {
        tags: Array.isArray(input.tags) ? input.tags : Array.isArray(original.tags) ? original.tags : [],
        aliases: Array.isArray(input.aliases) ? input.aliases : Array.isArray(original.aliases) ? original.aliases : [],
        id: original.id,
        webinyVersion: WEBINY_VERSION
      });
      try {
        await this.onFileBeforeUpdate.publish({
          original,
          file,
          input
        });
        const result = await storageOperations.files.update({
          original,
          file
        });
        await this.onFileAfterUpdate.publish({
          original,
          file,
          input
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update a file.", ex.code || "UPDATE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          original,
          file
        }));
      }
    },
    async deleteFile(id) {
      const permission = await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "d"
      });
      const file = await storageOperations.files.get({
        where: {
          id,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      });
      if (!file) {
        throw new _handlerGraphql.NotFoundError(`File with id "${id}" does not exists.`);
      }
      checkOwnership(file, permission, getIdentity());
      try {
        await this.onFileBeforeDelete.publish({
          file
        });
        await storageOperations.files.delete({
          file
        });
        await this.onFileAfterDelete.publish({
          file
        });
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete a file.", ex.code || "DELETE_FILE_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          id,
          file
        }));
      }
      return true;
    },
    async createFilesInBatch(inputs, meta) {
      await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "w"
      });
      const identity = getIdentity();
      const tenant = getTenantId();
      const locale = getLocaleCode();
      const createdBy = {
        id: identity.id,
        displayName: identity.displayName,
        type: identity.type
      };
      const files = inputs.map(input => {
        return (0, _objectSpread2.default)((0, _objectSpread2.default)({}, input), {}, {
          tags: Array.isArray(input.tags) ? input.tags : [],
          aliases: Array.isArray(input.aliases) ? input.aliases : [],
          meta: (0, _objectSpread2.default)({
            private: false
          }, input.meta || {}),
          tenant,
          createdOn: new Date().toISOString(),
          createdBy,
          locale,
          webinyVersion: WEBINY_VERSION
        });
      });
      try {
        await this.onFileBeforeBatchCreate.publish({
          files,
          meta
        });
        const results = await storageOperations.files.createBatch({
          files
        });
        await this.onFileAfterBatchCreate.publish({
          files,
          meta
        });
        return results;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create a batch of files.", ex.code || "CREATE_FILES_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          files
        }));
      }
    },
    async listFiles(params = {}) {
      const permission = await (0, _checkBasePermissions.checkBasePermissions)(getPermission, {
        rwd: "r"
      });
      const {
        limit = 40,
        search = "",
        types = [],
        tags = [],
        ids = [],
        after = null,
        where: initialWhere,
        sort: initialSort
      } = params;
      const where = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, initialWhere), {}, {
        private: false,
        locale: getLocaleCode(),
        tenant: getTenantId()
      });
      /**
       * Always override the createdBy received from the user, if any.
       */
      if (permission.own === true) {
        const identity = getIdentity();
        where.createdBy = identity.id;
      }
      /**
       * We need to map the old GraphQL definition to the new one.
       * That GQL definition is marked as deprecated.
       */
      /**
       * To have standardized where objects across the applications, we transform the types into type_in.
       */
      if (Array.isArray(types) && types.length > 0 && !where.type_in) {
        where.type_in = types;
      }
      /**
       * We are assigning search to tag and name search.
       * This should be treated as OR condition in the storage operations.
       */
      if (search && !where.search) {
        where.search = search;
      }
      /**
       * Same as on types/type_in.
       */
      if (Array.isArray(tags) && tags.length > 0 && !where.tag_in) {
        where.tag_in = tags.map(tag => tag.toLowerCase());
      }
      /**
       * Same as on types/type_in.
       */
      if (Array.isArray(ids) && ids.length > 0 && !where.id_in) {
        where.id_in = ids;
      }
      const sort = Array.isArray(initialSort) && initialSort.length > 0 ? initialSort : ["id_DESC"];
      try {
        return await storageOperations.files.list({
          where,
          after,
          limit,
          sort
        });
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list files by given parameters.", ex.code || "FILE_TAG_SEARCH_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          where,
          after,
          limit,
          sort
        }));
      }
    },
    async listTags({
      where: initialWhere,
      after,
      limit
    }) {
      await (0, _checkBasePermissions.checkBasePermissions)(getPermission);
      const where = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, initialWhere), {}, {
        tenant: getTenantId(),
        locale: getLocaleCode()
      });
      const params = {
        where,
        limit: limit || 1000000,
        after
      };
      try {
        return await storageOperations.files.tags(params);
      } catch (ex) {
        throw new _error.default(ex.message || "Could not search for tags.", ex.code || "FILE_TAG_SEARCH_ERROR", (0, _objectSpread2.default)((0, _objectSpread2.default)({}, ex.data || {}), {}, {
          params
        }));
      }
    }
  };
};
exports.createFilesCrud = createFilesCrud;